/*
 * test and example for options.h
 */
#include <stdio.h>
#include <string.h>
#include <options.h>

enum {
  BLACK,
  BLUE,
  GREEN,
  CYAN,
  RED,
  MAGENTA,
  BROWN,
  LIGHTGRAY,
  DARKGRAY,
  LIGHTBLUE,
  LIGHTGREEN,
  LIGHTCYAN,
  LIGHTRED,
  LIGHTMAGENTA,
  YELLOW,
  WHITE
};
char *color[] = {
  "BLACK",
  "BLUE",
  "GREEN",
  "CYAN",
  "RED",
  "MAGENTA",
  "BROWN",
  "LIGHTGRAY",
  "DARKGRAY",
  "LIGHTBLUE",
  "LIGHTGREEN",
  "LIGHTCYAN",
  "LIGHTRED",
  "LIGHTMAGENTA",
  "YELLOW",
  "WHITE",
};
char buf[256];

/*
 * global variables for examples
 */
char title[256] = "test program of options.h";
char name[256] = "options0";
char c = 'c';
int flag = !0;
int mode = !0;
int sw = 0;
int number = 0;
int width = 1152, height = 900;
float f = 0.001;
double d = 0.002;

/*
 * packed options
 */
option_record options[] = {
  { "-t %s",     title },
  { "-n \"%s\"", name  },
  { "-c %c",     &c    },
  { "-on[-]",    &flag },
  { "[-+]mode",  &mode },
  { "[+-]sw",    &sw   },
  { "-f %f",     &f    },
  { "-d %lf",    &d    },
  { "-help",     NULL  }
};

/*
 * to show variables for examples
 */
void usage(FILE *fp)
{
  fprintf(fp, "options ... \n");
  fprintf(fp, "\t-t name:    to set string (%s).   \n", title);
  fprintf(fp, "\t-n name:    to set string (%s).   \n", name);
  fprintf(fp, "\t-c char:    to set character (%c).\n", c);
  fprintf(fp, "\t-on[-]:     to set flag (%s).     \n", flag?"on":"off");
  fprintf(fp, "\t[-+]mode:   to set mode (%s).     \n", mode?"yes":"no");
  fprintf(fp, "\t[+-]sw:     to set switch (%s).   \n", sw?"on":"off");
  fprintf(fp, "\t-color enum to set enumerate (%s) \n", color[number]);
  fprintf(fp, "\t-geom WxH:  to set ints (%1dx%1d).\n", width, height);
  fprintf(fp, "\t-f float:   to set float (%f).    \n", f);
  fprintf(fp, "\t-d double:  to set double (%f).   \n", d);
  fprintf(fp, "\t-help:      show this help.       \n");
}

void main(int argc, char *argv[])
{
  int i;

  set_options(sizeof(options)/sizeof(option_record), options);
  set_option("-geom %dx%d", &width, &height);
  strcpy(buf, "-color ");
  for (i=0; i<16; i++) {
    strcat(buf, color[i]);
    strcat(buf, " ");
  }
  set_option(buf, &number);
  alias_option("-geom", "-geometry");

  printf("*** DEFAULT SETTINGS ***\n");
  usage(stdout);

  printf("*** COMMAND LINE ***\n");
  printf("\t");
  for (i=0; i<argc; i++)
    printf("%s ", argv[i]);
  printf("\n");

  scan_options(argc, argv);
  if (is_option("-help")) {
    printf("*** SETTINGS BY YOUR OPTIONS ***\n");
    usage(stdout);
    fprintf(stdout, "\tThere is help option.\n");
    exit(0);
  }

  printf("*** SETTINGS BY YOUR OPTIONS ***\n");
  usage(stdout);

  reset_options();

  exit(0);
}
